import logging
import os
import shlex
import subprocess
import time

from datetime import datetime
from pathlib import Path
from watchdog.events import FileSystemEventHandler


logging.basicConfig(
	format = '%(levelname)s: %(message)s'
)


class VenvWatcher(FileSystemEventHandler):
	#patterns = [f'*.{ext}' for ext in config.watch_ext]

	proc = None
	last_restart = None


	def __init__(self, env):
		super().__init__()

		self.env = env


	def on_any_event(self, event):
		if event.event_type not in ['modified', 'created', 'deleted']:
			logging.debug(f'Ignored event: {event.event_type}')
			return

		path = Path(event.src_path)
		directory = str(path.parent).replace(str(self.env.watcher.path), '')

		if not path.suffix:
			return

		if path.suffix[1:] not in self.env.watcher.ext:
			return

		if not set(self.env.watcher.ignore_dirs).intersection(directory.split('/')):
			for filename in self.env.watcher.ignore_files:
				if path.name == filename:
					logging.debug(f'Ignored file name: {filename}')
					return

			self.run_proc(restart=True)


	def kill_proc(self):
		if self.proc.poll() != None:
			return

		logging.info(f'Terminating process {self.proc.pid}')
		self.proc.terminate()
		sec = 0.0

		while self.proc.poll() == None:
			time.sleep(0.1)
			sec += 0.1

			if sec >= 5:
				logging.error('Failed to terminate. Killing process...')
				self.proc.kill()
				break

		logging.info('Process terminated')


	def run_proc(self, restart=False):
		timestamp = datetime.timestamp(datetime.now())
		self.last_restart = timestamp if not self.last_restart else 0

		if restart == True and self.proc.pid != '':
			if timestamp - 3 < self.last_restart:
				logging.debug('Process restarted less than 3 sec ago')
				return

			self.kill_proc()

		command = shlex.split(self.env.watcher.command)

		environment = dict(os.environ)
		environment.update(self.env.watcher.env)

		if Path(command[0]).is_file():
			self.proc = subprocess.Popen([self.env.executable, *command], env=environment)

		else:
			self.proc = subprocess.Popen([self.env.executable, '-m', *command], env=environment)

		self.last_restart = timestamp
		logging.info(f'Started process with PID {self.proc.pid}')
